/* Copyright c 2005-2012.
 * Licensed under GNU  LESSER General Public License, Version 3.
 * http://www.gnu.org/licenses
 */
package org.beangle.security.blueprint.service;

import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.beangle.security.blueprint.Authority;
import org.beangle.security.blueprint.Group;
import org.beangle.security.blueprint.Menu;
import org.beangle.security.blueprint.MenuProfile;
import org.beangle.security.blueprint.Resource;
import org.beangle.security.blueprint.User;

/**
 * 用户用户组权限管理服务接口. 权限实体@see <code>Authority</code> 系统资源实体@see <code>Resource</code> 系统功能点实体@see
 * <code>Action</code> 数据权限实体@see <code>Restriction</code>
 * 
 * @author dell,chaostone 2005-9-27
 */
public interface AuthorityService {

	/**
	 * 按照资源名称查询单独的资源
	 * 
	 * @param name
	 * @return
	 */
	public Resource getResource(String name);

	/**
	 * 查询用户的访问资源范围
	 * 
	 * @param user
	 * @return
	 */
	public List<Resource> getResources(User user);

	/**
	 * 查询指定可见范围的资源
	 * 
	 * @param scope
	 * @return
	 */
	public Set<String> getResourceNames(int scope);

	/**
	 * 按照用户组查找资源
	 * 
	 * @param group
	 * @return
	 */
	public Set<String> getResourceNamesByGroup(String group);

	/**
	 * 用户组内对应的资源
	 * 
	 * @param group
	 * @return
	 */
	public List<Resource> getResources(Group group);

	/**
	 * 更新资源状态
	 * 
	 * @param resourceIds
	 * @param isEnabled
	 */
	public void updateState(Long[] resourceIds, boolean isEnabled);

	/**
	 * 依据默认深度（小于或等于）得到用户的所有权限
	 * 
	 * @param user
	 * @return
	 */
	public List<Authority> getAuthorities(User user);

	/**
	 * 依据默认深度得到用户组拥有的权限
	 * 
	 * @param group
	 * @return
	 */
	public List<Authority> getAuthorities(Group group);

	/**
	 * 依据资源和用户取得对应的权限
	 * 
	 * @param user
	 * @param resource
	 * @return
	 */
	public Authority getAuthority(User user, Resource resource);

	/**
	 * 依据资源和用户组取得对应的权限
	 * 
	 * @param userId
	 * @param resource
	 * @return
	 */
	public Authority getAuthority(Group group, Resource resource);

	/**
	 * 获取用户的直接权限范围内的资源和所具有用户组的资源.
	 * 
	 * @param userId
	 * @return
	 */
	public List<Menu> getMenus(MenuProfile profile, User user);

	/**
	 * 用户组内对应的资源
	 * 
	 * @param groupId
	 * @return
	 */
	public List<Menu> getMenus(MenuProfile profile, Group group, Boolean enabled);

	/**
	 * 保存新的权限设置
	 * 
	 * @param authority
	 */
	public void saveOrUpdate(Authority authority);

	/**
	 * 删除权限的信息设置
	 * 
	 * @param authority
	 */
	public void remove(Authority authority);

	/**
	 * 授权
	 * 
	 * @param ao
	 * @param resources
	 * @param authorityClass
	 */
	public void authorize(Group group, Set<Resource> resources);

	/**
	 * 
	 * @param userService
	 */
	public void setUserService(UserService userService);

	/**
	 * 
	 * @return
	 */
	public UserService getUserService();

	/**
	 * Extract Resource from uri
	 * @param uri with out context path
	 * @return
	 */
	public String extractResource(String uri);

	/**
	 * 过滤有该资源权限的用户组
	 * @param groups
	 * @param resource
	 * @return
	 */
	public Collection<Group> filter(Collection<Group> groups, Resource resource);
}
